class Lazy:
    """
    class A:
        @Lazy
        def answer_to_life(self):
            return expensive_calculation

    obj = A()
    print(obj.answer_to_life) # will calculate and store
    print(obj.answer_to_life) # will just look up stored value
    """

    def __init__(self, calculate_function) -> None:
        self._calculate = calculate_function

    def __get__(self, obj, _=None):
        if obj is None:
            return self
        value = self._calculate(obj)
        setattr(obj, self._calculate.__name__, value)
        return value



from . import service_config


def test_production_false(monkeypatch):
    monkeypatch.setenv('SL_ENVIRONMENT', 'test')
    assert service_config.is_production() is False


def test_production_true(monkeypatch):
    monkeypatch.setenv('SL_ENVIRONMENT', 'prod')
    assert service_config.is_production() is True


def test_stage_false(monkeypatch):
    monkeypatch.setenv('SL_ENVIRONMENT', 'test')
    assert service_config.is_stage() is False


def test_stage_true(monkeypatch):
    monkeypatch.setenv('SL_ENVIRONMENT', 'stage')
    assert service_config.is_stage() is True


def test_version(monkeypatch):
    monkeypatch.setenv('SL_VERSION', 'recruiter-1912/dev:0.0.1')
    assert service_config.get_version() == 'recruiter-1912/dev:0.0.1'


def test_version_short(monkeypatch):
    monkeypatch.setenv('SL_VERSION', 'recruiter-1912/dev:0.0.1')
    assert service_config.get_version_short() == 'dev:0.0.1'


def test_version_short_none(monkeypatch):
    monkeypatch.delenv('SL_VERSION')
    assert service_config.get_version_short() is None


def test_get_info(monkeypatch):
    monkeypatch.setenv('SL_VERSION', 'recruiter-1912/dev:0.0.1')
    monkeypatch.setenv('SL_ENVIRONMENT', 'test')
    assert service_config.get_info() == {'version': 'recruiter-1912/dev:0.0.1', 'environment': 'test'}



from typing import Dict
from typing import Optional

import os


def is_production() -> bool:
    return _get_environment() == 'prod'


def is_stage() -> bool:
    return _get_environment() == 'stage'


def get_version() -> Optional[str]:
    return os.environ.get('SL_VERSION')


def get_version_short() -> Optional[str]:
    version = get_version()
    if version is None:
        return version
    image = version.split('/')[-1]
    return image


def get_info() -> Dict[str, Optional[str]]:
    return {'version': get_version(), 'environment': _get_environment()}


def _get_environment() -> Optional[str]:
    return os.environ.get('SL_ENVIRONMENT')
